@@ -45,3 +45,9 @@ def setup(app):
     app.add_config_value('edit_on_github_branch', 'master', True)
     app.add_config_value('edit_on_github_src_path', '', True)  # 'eg' "docs/"
     app.connect('html-page-context', html_page_context)
+    
+    return {
+        'version': '0.1',
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
